from ssl import SSLCertVerificationError, SSLError
import httpx
import tiktoken
import lxml.html
from lxml.etree import ParserError
from databeakers import Pipeline
from databeakers.http import HttpRequest, HttpResponse
from scrapeghost import SchemaScraper
from scrapeghost.preprocessors import CleanHTML
from .models import Agency, URL, Int
import csv


class CSVSource:
    def __init__(self, filename, datatype):
        self.filename = filename
        self.datatype = datatype

    def __call__(self):
        with open(self.filename) as inf:
            reader = csv.DictReader(inf)
            for line in reader:
                yield self.datatype(**line)


def tiktoken_count(response):
    if response["status_code"] != 200:
        raise ValueError("response status code is not 200")

    html = response["response_body"]

    # clean the html
    cleaner = CleanHTML()
    encoding = tiktoken.get_encoding("cl100k_base")
    doc = lxml.html.fromstring(html)
    (doc,) = cleaner(doc)  # returns a 1-item list
    html_again = lxml.html.tostring(doc, encoding="unicode")
    tokens = len(encoding.encode(html_again))

    response["tiktoken_count"] = tokens

    return response


# current thinking, beakers exist within a recipe
recipe = Pipeline("foiaghost", "foiaghost.db")
recipe.add_beaker("agency", Agency)
recipe.add_beaker("good_urls", URL)
recipe.add_transform("agency", "good_urls", lambda x: x["url"].startswith("http"))
recipe.add_beaker("responses", HttpResponse)
recipe.add_transform("good_urls", "responses", HttpRequest)
recipe.add_beaker("tiktoken_count", Int)
recipe.add_transform(
    "responses",
    "tiktoken_count",
    tiktoken_count,
    error_map={(ValueError, ParserError): "no_tiktoken_count"},
)
recipe.add_seed(
    "agencies",
    "agency",
    CSVSource("agencies.csv", Agency),
)

# recipe.add_beaker("token_lt_8k", temp=True)
# recipe.add_beaker("token_gt_8k", temp=True)


# recipe.add_transform(
#     "good_urls",
#     "responses",
#     add_response,
#     error_map={
#         (
#             httpx.HTTPError,
#             SSLCertVerificationError,
#             SSLError,
#         ): "bad_requests"
#     },
# )
# recipe.add_conditional(
#     "with_tiktoken_count",
#     lambda x: x["tiktoken_count"] < 8000,
#     if_true="token_lt_8k",
#     if_false="token_gt_8k",
# )
